<?php

namespace Gems\Student\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Gems\Student\Http\Requests\StudentRequest;
use Gems\Student\Interfaces\StudentRepositoryInterface;
use Gems\Student\Models\Student;

/**
 * Resource controller class for student.
 */
class StudentResourceController extends BaseController
{

    /**
     * Initialize student resource controller.
     *
     * @param type StudentRepositoryInterface $student
     *
     * @return null
     */
    public function __construct(StudentRepositoryInterface $student)
    {
        parent::__construct();
        $this->repository = $student;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Gems\Student\Repositories\Criteria\StudentResourceCriteria::class);
    }

    /**
     * Display a list of student.
     *
     * @return Response
     */
    public function index(StudentRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Gems\Student\Repositories\Presenter\StudentPresenter::class)
                ->$function();
        }

        $students = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('student::student.names'))
            ->view('student::student.index', true)
            ->data(compact('students', 'view'))
            ->output();
    }

    /**
     * Display student.
     *
     * @param Request $request
     * @param Model   $student
     *
     * @return Response
     */
    public function show(StudentRequest $request, Student $student)
    {

        if ($student->exists) {
            $view = 'student::student.show';
        } else {
            $view = 'student::student.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('student::student.name'))
            ->data(compact('student'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new student.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(StudentRequest $request)
    {

        $student = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('student::student.name')) 
            ->view('student::student.create', true) 
            ->data(compact('student'))
            ->output();
    }

    /**
     * Create new student.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StudentRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $student                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('student::student.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('student/student/' . $student->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/student/student'))
                ->redirect();
        }

    }

    /**
     * Show student for editing.
     *
     * @param Request $request
     * @param Model   $student
     *
     * @return Response
     */
    public function edit(StudentRequest $request, Student $student)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('student::student.name'))
            ->view('student::student.edit', true)
            ->data(compact('student'))
            ->output();
    }

    /**
     * Update the student.
     *
     * @param Request $request
     * @param Model   $student
     *
     * @return Response
     */
    public function update(StudentRequest $request, Student $student)
    {
        try {
            $attributes = $request->all();

            $student->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('student::student.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('student/student/' . $student->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('student/student/' . $student->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the student.
     *
     * @param Model   $student
     *
     * @return Response
     */
    public function destroy(StudentRequest $request, Student $student)
    {
        try {

            $student->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('student::student.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('student/student/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('student/student/' . $student->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple student.
     *
     * @param Model   $student
     *
     * @return Response
     */
    public function delete(StudentRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('student::student.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('student/student'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/student/student'))
                ->redirect();
        }

    }

    /**
     * Restore deleted students.
     *
     * @param Model   $student
     *
     * @return Response
     */
    public function restore(StudentRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('student::student.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/student/student'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/student/student/'))
                ->redirect();
        }

    }

}
